/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KQTISECTIONPART_H
#define KQTISECTIONPART_H

#include <kqti_export.h>
#include <QDomElement>

/**
 * \class KQtiSectionPart
 * Sections group together individual item references and/or sub-sections. A
 * number of common parameters are shared by both types of child element.
 * 
 * KQtiSectionPart is an abstract class providing an common interface for
 * KQtiAssessmentSections, KQtiAssessmentItemRefs and includes.
 * 
 * Abstract class
 * http://www.imsglobal.org/question/qtiv2p1pd2/imsqti_infov2p1pd2.html#element10503
 */

class KQTI_EXPORT KQtiSectionPart /*This is a base class*/
{
    public:
        KQtiSectionPart();
        KQtiSectionPart(const QDomElement& element);
        virtual ~KQtiSectionPart();
        
        void setIdentifier(const QString& identifier) {mIdentifier = identifier;}
        QString identifier() const {return mIdentifier;}
        
        /**
         * element
         * @return create and return the DOM element
         */
        virtual QDomElement element() const = 0;
        
    protected:
        QString mIdentifier;
        //itemSessionControl //[0..1] TODO This exists now
};

#endif
