/***************************************************************************
 *   Copyright (C) 2010 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "veneer/contextmenu.h"

#include <QAction>
#include <QWidgetAction>
#include <QEvent>
#include <QToolButton>
#include <QDebug>

namespace Veneer
{

	ContextMenu::ContextMenu() :
		QMenu()
	{
	}

	bool ContextMenu::eventFilter(QObject*, QEvent* event)
	{
		if (event->type() == QEvent::Paint
				|| event->type() == QEvent::KeyPress
				|| event->type() == QEvent::KeyRelease)
			return false;

		event->accept();

		return true;
	}

	QAction* ContextMenu::addTitle(const QString& title)
	{
		if (title.isEmpty())
			return NULL;

		QAction* buttonAction = new QAction(title, this);

		QFont font = buttonAction->font();
		font.setBold(true);
		buttonAction->setFont(font);

		QWidgetAction* titleAction = new QWidgetAction(this);

		QToolButton* titleButton = new QToolButton(this);
		titleButton->setDefaultAction(buttonAction);

		titleButton->setDown(true);
		titleButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
		titleAction->setDefaultWidget(titleButton);

		titleButton->installEventFilter(this);

		insertAction(NULL, titleAction);

		return titleAction;
	}

}
